@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Robey Pointer <robey@lag.net>
+# Copyright (C) 2006-2007  Robey Pointer <robey@lag.net>
 #
 # This file is part of paramiko.
 #
@@ -23,11 +23,14 @@ L{SSHClient}.
 from binascii import hexlify
 import getpass
 import os
+import socket
+import warnings
 
 from paramiko.agent import Agent
 from paramiko.common import *
 from paramiko.dsskey import DSSKey
 from paramiko.hostkeys import HostKeys
+from paramiko.resource import ResourceManager
 from paramiko.rsakey import RSAKey
 from paramiko.ssh_exception import SSHException, BadHostKeyException
 from paramiko.transport import Transport
@@ -79,7 +82,17 @@ class RejectPolicy (MissingHostKeyPolicy):
                     (key.get_name(), hostname, hexlify(key.get_fingerprint())))
         raise SSHException('Unknown server %s' % hostname)
 
-        
+
+class WarningPolicy (MissingHostKeyPolicy):
+    """
+    Policy for logging a python-style warning for an unknown host key, but
+    accepting it. This is used by L{SSHClient}.
+    """
+    def missing_host_key(self, client, hostname, key):
+        warnings.warn('Unknown %s host key for %s: %s' %
+                      (key.get_name(), hostname, hexlify(key.get_fingerprint())))
+
+
 class SSHClient (object):
     """
     A high-level representation of a session with an SSH server.  This class
@@ -211,7 +224,7 @@ class SSHClient (object):
         self._policy = policy
 
     def connect(self, hostname, port=22, username=None, password=None, pkey=None,
-                key_filename=None):
+                key_filename=None, timeout=None, allow_agent=True, look_for_keys=True):
         """
         Connect to an SSH server and authenticate to it.  The server's host key
         is checked against the system host keys (see L{load_system_host_keys})
@@ -242,20 +255,38 @@ class SSHClient (object):
         @type password: str
         @param pkey: an optional private key to use for authentication
         @type pkey: L{PKey}
-        @param key_filename: the filename of an optional private key to use
-            for authentication
-        @type key_filename: str
-        
+        @param key_filename: the filename, or list of filenames, of optional
+            private key(s) to try for authentication
+        @type key_filename: str or list(str)
+        @param timeout: an optional timeout (in seconds) for the TCP connect
+        @type timeout: float
+        @param allow_agent: set to False to disable connecting to the SSH agent
+        @type allow_agent: bool
+        @param look_for_keys: set to False to disable searching for discoverable
+            private key files in C{~/.ssh/}
+        @type look_for_keys: bool
+
         @raise BadHostKeyException: if the server's host key could not be
             verified
         @raise AuthenticationException: if authentication failed
         @raise SSHException: if there was any other error connecting or
             establishing an SSH session
+        @raise socket.error: if a socket error occurred while connecting
         """
-        t = self._transport = Transport((hostname, port))
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        if timeout is not None:
+            try:
+                sock.settimeout(timeout)
+            except:
+                pass
+
+        sock.connect((hostname, port))
+        t = self._transport = Transport(sock)
+
         if self._log_channel is not None:
             t.set_log_channel(self._log_channel)
         t.start_client()
+        ResourceManager.register(self, t)
         
         server_key = t.get_remote_server_key()
         keytype = server_key.get_name()
@@ -274,16 +305,25 @@ class SSHClient (object):
 
         if username is None:
             username = getpass.getuser()
-        self._auth(username, password, pkey, key_filename)
+        
+        if key_filename is None:
+            key_filenames = []
+        elif isinstance(key_filename, (str, unicode)):
+            key_filenames = [ key_filename ]
+        else:
+            key_filenames = key_filename
+        self._auth(username, password, pkey, key_filenames, allow_agent, look_for_keys)
     
     def close(self):
         """
         Close this SSHClient and its underlying L{Transport}.
         """
+        if self._transport is None:
+            return
         self._transport.close()
         self._transport = None
 
-    def exec_command(self, command):
+    def exec_command(self, command, bufsize=-1):
         """
         Execute a command on the SSH server.  A new L{Channel} is opened and
         the requested command is executed.  The command's input and output
@@ -292,6 +332,8 @@ class SSHClient (object):
         
         @param command: the command to execute
         @type command: str
+        @param bufsize: interpreted the same way as by the built-in C{file()} function in python
+        @type bufsize: int
         @return: the stdin, stdout, and stderr of the executing command
         @rtype: tuple(L{ChannelFile}, L{ChannelFile}, L{ChannelFile})
 
@@ -299,9 +341,9 @@ class SSHClient (object):
         """
         chan = self._transport.open_session()
         chan.exec_command(command)
-        stdin = chan.makefile('wb')
-        stdout = chan.makefile('rb')
-        stderr = chan.makefile_stderr('rb')
+        stdin = chan.makefile('wb', bufsize)
+        stdout = chan.makefile('rb', bufsize)
+        stderr = chan.makefile_stderr('rb', bufsize)
         return stdin, stdout, stderr
 
     def invoke_shell(self, term='vt100', width=80, height=24):
@@ -334,14 +376,25 @@ class SSHClient (object):
         @rtype: L{SFTPClient}
         """
         return self._transport.open_sftp_client()
+    
+    def get_transport(self):
+        """
+        Return the underlying L{Transport} object for this SSH connection.
+        This can be used to perform lower-level tasks, like opening specific
+        kinds of channels.
         
-    def _auth(self, username, password, pkey, key_filename):
+        @return: the Transport for this connection
+        @rtype: L{Transport}
+        """
+        return self._transport
+        
+    def _auth(self, username, password, pkey, key_filenames, allow_agent, look_for_keys):
         """
         Try, in order:
         
             - The key passed in, if one was passed in.
-            - Any key we can find through an SSH agent.
-            - Any "id_rsa" or "id_dsa" key discoverable in ~/.ssh/.
+            - Any key we can find through an SSH agent (if allowed).
+            - Any "id_rsa" or "id_dsa" key discoverable in ~/.ssh/ (if allowed).
             - Plain username/password auth, if a password was given.
         
         (The password might be needed to unlock a private key.)
@@ -356,7 +409,7 @@ class SSHClient (object):
             except SSHException, e:
                 saved_exception = e
 
-        if key_filename is not None:
+        for key_filename in key_filenames:
             for pkey_class in (RSAKey, DSSKey):
                 try:
                     key = pkey_class.from_private_key_file(key_filename, password)
@@ -365,24 +418,35 @@ class SSHClient (object):
                     return
                 except SSHException, e:
                     saved_exception = e
-
-        for key in Agent().get_keys():
-            try:
-                self._log(DEBUG, 'Trying SSH agent key %s' % hexlify(key.get_fingerprint()))
-                self._transport.auth_publickey(username, key)
-                return
-            except SSHException, e:
-                saved_exception = e
+        
+        if allow_agent:
+            for key in Agent().get_keys():
+                try:
+                    self._log(DEBUG, 'Trying SSH agent key %s' % hexlify(key.get_fingerprint()))
+                    self._transport.auth_publickey(username, key)
+                    return
+                except SSHException, e:
+                    saved_exception = e
 
         keyfiles = []
         rsa_key = os.path.expanduser('~/.ssh/id_rsa')
-        dsa_key = os.path.expanduser('~/.ssh/is_dsa')
+        dsa_key = os.path.expanduser('~/.ssh/id_dsa')
         if os.path.isfile(rsa_key):
             keyfiles.append((RSAKey, rsa_key))
         if os.path.isfile(dsa_key):
-            keyfiles.append((DSSKey, dss_key))
+            keyfiles.append((DSSKey, dsa_key))
+        # look in ~/ssh/ for windows users:
+        rsa_key = os.path.expanduser('~/ssh/id_rsa')
+        dsa_key = os.path.expanduser('~/ssh/id_dsa')
+        if os.path.isfile(rsa_key):
+            keyfiles.append((RSAKey, rsa_key))
+        if os.path.isfile(dsa_key):
+            keyfiles.append((DSSKey, dsa_key))
+        
+        if not look_for_keys:
+            keyfiles = []
+
         for pkey_class, filename in keyfiles:
-            filename = os.path.expanduser('~/.ssh/' + filename)
             try:
                 key = pkey_class.from_private_key_file(filename, password)
                 self._log(DEBUG, 'Trying discovered key %s in %s' % (hexlify(key.get_fingerprint()), filename))
