@@ -1,6 +1,4 @@
-#!/usr/bin/python
-
-# Copyright (C) 2003-2005 Robey Pointer <robey@lag.net>
+# Copyright (C) 2003-2007  Robey Pointer <robey@lag.net>
 #
 # This file is part of paramiko.
 #
@@ -61,6 +59,9 @@ from paramiko import *
 
 class UtilTest (unittest.TestCase):
 
+    assertTrue = unittest.TestCase.failUnless   # for Python 2.3 and below
+    assertFalse = unittest.TestCase.failIf      # for Python 2.3 and below
+
     def setUp(self):
         pass
 
@@ -77,6 +78,7 @@ class UtilTest (unittest.TestCase):
         self.assertTrue('MissingHostKeyPolicy' in symbols)
         self.assertTrue('AutoAddPolicy' in symbols)
         self.assertTrue('RejectPolicy' in symbols)
+        self.assertTrue('WarningPolicy' in symbols)
         self.assertTrue('SecurityOptions' in symbols)
         self.assertTrue('SubsystemHandler' in symbols)
         self.assertTrue('Channel' in symbols)
@@ -144,3 +146,9 @@ class UtilTest (unittest.TestCase):
         finally:
             os.unlink('hostfile.temp')
 
+    def test_6_random(self):
+        from paramiko.common import randpool
+        # just verify that we can pull out 32 bytes and not get an exception.
+        x = randpool.get_bytes(32)
+        self.assertEquals(len(x), 32)
+        
